@@ -78,7 +78,20 @@ ARDOUR_UI::setup_windows ()
 
 	theme_manager->signal_unmap().connect (bind (sigc::ptr_fun(&ActionManager::uncheck_toggleaction), X_("<Actions>/Common/ToggleThemeManager")));
 
+#ifdef TOP_MENUBAR
+	HBox* status_bar_packer = manage (new HBox);
+	
+	status_bar_label.set_size_request (300, -1);
+	status_bar_packer->pack_start (status_bar_label, true, true, 6);
+	status_bar_packer->pack_start (error_log_button, false, false);
+	
+	error_log_button.signal_clicked().connect (mem_fun (*this, &UI::toggle_errors));
+
+	editor->get_status_bar_packer().pack_start (*status_bar_packer, true, true);
+	editor->get_status_bar_packer().pack_start (menu_bar_base, false, false, 6);
+#else
 	top_packer.pack_start (menu_bar_base, false, false);
+#endif 
 	top_packer.pack_start (transport_frame, false, false);
 
 	editor->add_toplevel_controls (top_packer);
@@ -86,6 +99,32 @@ ARDOUR_UI::setup_windows ()
 	return 0;
 }
 
+void
+ARDOUR_UI::display_message (const char *prefix, gint prefix_len, RefPtr<TextBuffer::Tag> ptag, RefPtr<TextBuffer::Tag> mtag, const char *msg)
+{
+	ustring text;
+
+	UI::display_message (prefix, prefix_len, ptag, mtag, msg);
+#ifdef TOP_MENUBAR
+
+	if (strcmp (prefix, _("[ERROR]: ")) == 0) {
+		text = "<span color=\"red\" weight=\"bold\">";
+	} else if (strcmp (prefix, _("[WARNING]: ")) == 0) {
+		text = "<span color=\"yellow\" weight=\"bold\">";
+	} else if (strcmp (prefix, _("[INFO]: ")) == 0) {
+		text = "<span color=\"green\" weight=\"bold\">";
+	} else {
+		text = "<span color=\"blue\" weight=\"bold\">???";
+	}
+
+	text += prefix;
+	text += "</span>";
+	text += msg;
+
+	status_bar_label.set_markup (text);
+#endif
+}
+
 void
 ARDOUR_UI::transport_stopped ()
 {
@@ -276,6 +315,7 @@ ARDOUR_UI::setup_transport ()
 
 	primary_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::primary_clock_value_changed));
 	secondary_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::secondary_clock_value_changed));
+	big_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::big_clock_value_changed));
 
 	ARDOUR_UI::instance()->tooltips().set_tip (primary_clock, _("Primary clock"));
 	ARDOUR_UI::instance()->tooltips().set_tip (secondary_clock, _("secondary clock"));
@@ -358,7 +398,9 @@ ARDOUR_UI::setup_transport ()
 	transport_tearoff_hbox.pack_start (*svbox, false, false, 3);
 
 	transport_tearoff_hbox.pack_start (auto_loop_button, false, false);
-	transport_tearoff_hbox.pack_start (play_selection_button, false, false);
+	if (!Profile->get_sae()) {
+		transport_tearoff_hbox.pack_start (play_selection_button, false, false);
+	}
 	transport_tearoff_hbox.pack_start (roll_button, false, false);
 	transport_tearoff_hbox.pack_start (stop_button, false, false);
 	transport_tearoff_hbox.pack_start (rec_button, false, false, 6);
@@ -368,12 +410,16 @@ ARDOUR_UI::setup_transport ()
 	if (!ARDOUR::Profile->get_small_screen()) {
 		clock_box->pack_start (secondary_clock, false, false);
 	}
-	VBox* time_controls_box = manage (new VBox);
-	time_controls_box->pack_start (sync_option_combo, false, false);
-	time_controls_box->pack_start (time_master_button, false, false);
-	clock_box->pack_start (*time_controls_box, false, false, 1);
+
+	if (!Profile->get_sae()) {
+		VBox* time_controls_box = manage (new VBox);
+		time_controls_box->pack_start (sync_option_combo, false, false);
+		time_controls_box->pack_start (time_master_button, false, false);
+		clock_box->pack_start (*time_controls_box, false, false, 1);
+	}
+
 	transport_tearoff_hbox.pack_start (*clock_box, false, false, 0);
-	
+
 	HBox* toggle_box = manage(new HBox);
 	
 	VBox* punch_box = manage (new VBox);
@@ -403,6 +449,11 @@ ARDOUR_UI::setup_transport ()
 
 	transport_tearoff_hbox.pack_start (*toggle_box, false, false, 4);
 	transport_tearoff_hbox.pack_start (alert_box, false, false);
+
+	if (Profile->get_sae()) {
+		Image* img = manage (new Image ((::get_icon (X_("sae")))));
+		transport_tearoff_hbox.pack_end (*img, false, false, 6);
+	}
 }
 
 void
@@ -835,7 +886,10 @@ void
 ARDOUR_UI::sync_option_changed ()
 {
 	if (session) {
-		session->request_slave_source (string_to_slave_source (sync_option_combo.get_active_text()));
+		ustring txt = sync_option_combo.get_active_text ();
+		if (txt.length()) {
+			session->request_slave_source (string_to_slave_source (txt));
+		}
 	}
 }
 
