@@ -855,11 +855,12 @@ AudioClock::field_key_release_event (GdkEventKey *ev, Field field)
 			case Bars:
 			case Beats:
 			case Ticks:
-				// Bars or beats should never be 0
-				if (atoi(bars_label.get_text()) == 0) {
+				// Bars should never be, unless this clock is for a duration
+				if (atoi(bars_label.get_text()) == 0 && !is_duration) {
 					bars_label.set_text("001");
 				}
-				if (atoi(beats_label.get_text()) == 0) {
+				//  beats should never be 0, unless this clock is for a duration
+				if (atoi(beats_label.get_text()) == 0 && !is_duration) {
 					beats_label.set_text("01");
 				}
 				break;
