@@ -46,7 +46,6 @@
 #include "rgb_macros.h"
 #include "gui_thread.h"
 #include "utils.h"
-#include "color.h"
 
 #include "i18n.h"
 
@@ -62,12 +61,12 @@ AudioStreamView::AudioStreamView (AudioTimeAxisView& tv)
 	_waveform_shape = Traditional;
 	
 	if (tv.is_audio_track())
-		stream_base_color = color_map[cAudioTrackBase];
+		stream_base_color = ARDOUR_UI::config()->canvasvar_AudioTrackBase.get();
 	else
-		stream_base_color = color_map[cAudioBusBase];
+		stream_base_color = ARDOUR_UI::config()->canvasvar_AudioBusBase.get();
 	
 	canvas_rect->property_fill_color_rgba() = stream_base_color;
-	canvas_rect->property_outline_color_rgba() = color_map[cAudioTrackOutline];
+	canvas_rect->property_outline_color_rgba() = RGBA_BLACK;
 
 	_amplitude_above_axis = 1.0;
 
@@ -279,9 +278,15 @@ AudioStreamView::playlist_modified ()
 }
 
 void
-AudioStreamView::playlist_changed (boost::shared_ptr<Diskstream> ds)
+AudioStreamView::playlist_changed (boost::weak_ptr<Diskstream> wptr)
 {
-	ENSURE_GUI_THREAD (bind (mem_fun (*this, &AudioStreamView::playlist_changed), ds));
+	ENSURE_GUI_THREAD (bind (mem_fun (*this, &AudioStreamView::playlist_changed), wptr));
+
+	boost::shared_ptr<Diskstream> ds  = wptr.lock();
+
+	if (!ds) {
+		return;
+	}
 
 	StreamView::playlist_changed(ds);
 
@@ -508,24 +513,22 @@ AudioStreamView::setup_rec_box ()
 			
 			/* start a new rec box */
 
-			AudioTrack* at;
-
-			at = _trackview.audio_track(); /* we know what it is already */
+			boost::shared_ptr<AudioTrack> at = _trackview.audio_track ();
 			boost::shared_ptr<AudioDiskstream> ds = at->audio_diskstream();
 			nframes_t frame_pos = ds->current_capture_start ();
 			gdouble xstart = _trackview.editor.frame_to_pixel (frame_pos);
 			gdouble xend;
 			uint32_t fill_color;
 
-			switch (_trackview.audio_track()->mode()) {
+			switch (at->mode()) {
 			case Normal:
 				xend = xstart;
-				fill_color = color_map[cRecordingRectFill];
+				fill_color = ARDOUR_UI::config()->canvasvar_RecordingRect.get();
 				break;
 
 			case Destructive:
 				xend = xstart + 2;
-				fill_color = color_map[cRecordingRectFill];
+				fill_color = ARDOUR_UI::config()->canvasvar_RecordingRect.get();
 				/* make the recording rect translucent to allow
 				   the user to see the peak data coming in, etc.
 				*/
@@ -538,7 +541,7 @@ AudioStreamView::setup_rec_box ()
 			rec_rect->property_y1() = 1.0;
 			rec_rect->property_x2() = xend;
 			rec_rect->property_y2() = (double) _trackview.height - 1;
-			rec_rect->property_outline_color_rgba() = color_map[cRecordingRectOutline];
+			rec_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RecordingRect.get();
 			rec_rect->property_fill_color_rgba() = fill_color;
 			
 			RecBoxInfo recbox;
@@ -761,25 +764,15 @@ AudioStreamView::reveal_xfades_involving (AudioRegionView& rv)
 }
 
 void
-AudioStreamView::color_handler (ColorID id, uint32_t val)
+AudioStreamView::color_handler ()
 {
-	switch (id) {
-	case cAudioTrackBase:
-		if (_trackview.is_track()) {
-			canvas_rect->property_fill_color_rgba() = val;
-		} 
-		break;
-	case cAudioBusBase:
-		if (!_trackview.is_track()) {
-			canvas_rect->property_fill_color_rgba() = val;
-		}
-		break;
-	case cAudioTrackOutline:
-		canvas_rect->property_outline_color_rgba() = val;
-		break;
-
-	default:
-		break;
+	if (_trackview.is_track()) {
+		canvas_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_AudioTrackBase.get();
 	}
+
+	if (!_trackview.is_track()) {
+		canvas_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_AudioBusBase.get();
+	}
+ 
 }
 
