@@ -293,7 +293,8 @@ Editor::mouse_add_new_tempo_event (nframes_t frame)
 	TempoDialog tempo_dialog (map, frame, _("add"));
 	
 	tempo_dialog.set_position (Gtk::WIN_POS_MOUSE);
-	tempo_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &tempo_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
+	//this causes compiz to display no border.
+	//tempo_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &tempo_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
 
 	ensure_float (tempo_dialog);
 
@@ -320,7 +321,7 @@ Editor::mouse_add_new_tempo_event (nframes_t frame)
 	session->add_command(new MementoCommand<TempoMap>(map, &before, &after));
 	commit_reversible_command ();
 	
-	map.dump (cerr);
+	//map.dump (cerr);
 }
 
 void
@@ -335,7 +336,9 @@ Editor::mouse_add_new_meter_event (nframes_t frame)
 	MeterDialog meter_dialog (map, frame, _("add"));
 
 	meter_dialog.set_position (Gtk::WIN_POS_MOUSE);
-	meter_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &meter_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
+
+	//this causes compiz to display no border.. 
+	//meter_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &meter_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
 
 	ensure_float (meter_dialog);
 	
@@ -351,16 +354,16 @@ Editor::mouse_add_new_meter_event (nframes_t frame)
 	
 	double note_type = meter_dialog.get_note_type ();
 	BBT_Time requested;
-	
+
 	meter_dialog.get_bbt_time (requested);
-	
+
 	begin_reversible_command (_("add meter mark"));
         XMLNode &before = map.get_state();
 	map.add_meter (Meter (bpb, note_type), requested);
 	session->add_command(new MementoCommand<TempoMap>(map, &before, &map.get_state()));
 	commit_reversible_command ();
 	
-	map.dump (cerr);
+	//map.dump (cerr);
 }
 
 void
@@ -435,10 +438,14 @@ Editor::edit_tempo_section (TempoSection* section)
 	tempo_dialog.get_bbt_time(when);
 	bpm = max (0.01, bpm);
 	
+	cerr << "Editing tempo section to be at " << when << endl;
+	session->tempo_map().dump (cerr);
 	begin_reversible_command (_("replace tempo mark"));
         XMLNode &before = session->tempo_map().get_state();
 	session->tempo_map().replace_tempo (*section, Tempo (bpm,nt));
+	session->tempo_map().dump (cerr);
 	session->tempo_map().move_tempo (*section, when);
+	session->tempo_map().dump (cerr);
         XMLNode &after = session->tempo_map().get_state();
 	session->add_command (new MementoCommand<TempoMap>(session->tempo_map(), &before, &after));
 	commit_reversible_command ();
