@@ -21,6 +21,7 @@
 
 #include <algorithm>
 #include <fstream>
+#include <iostream>
 
 #include <ctype.h>
 
@@ -44,10 +45,10 @@ guint Keyboard::delete_mod = GDK_SHIFT_MASK;
 guint Keyboard::snap_mod = GDK_MOD3_MASK;
 
 #ifdef GTKOSX
-guint Keyboard::PrimaryModifier = GDK_MOD1_MASK;   // Command
-guint Keyboard::SecondaryModifier = GDK_MOD5_MASK; // Alt/Option
+guint Keyboard::PrimaryModifier = GDK_META_MASK;   // Command
+guint Keyboard::SecondaryModifier = GDK_MOD1_MASK; // Alt/Option
 guint Keyboard::TertiaryModifier = GDK_SHIFT_MASK; // Shift
-guint Keyboard::CopyModifier = GDK_MOD5_MASK;      // Alt/Option
+guint Keyboard::CopyModifier = GDK_MOD1_MASK;      // Alt/Option
 guint Keyboard::RangeSelectModifier = GDK_SHIFT_MASK;   
 #else
 guint Keyboard::PrimaryModifier = GDK_CONTROL_MASK; // Control
