@@ -59,16 +59,6 @@ extern int curvetest (string);
 static ARDOUR_UI  *ui = 0;
 static char* localedir = LOCALEDIR;
 
-gint
-show_ui_callback (void *arg)
-{
- 	ARDOUR_UI * ui = (ARDOUR_UI *) arg;
-
-	ui->hide_splash();
-	
-	return FALSE;
-}
-
 #ifdef __APPLE__
 
 #include <mach-o/dyld.h>
@@ -133,6 +123,36 @@ fixup_bundle_environment ()
 	
 	setenv ("LADSPA_PATH", path.c_str(), 1);
 
+	cstr = getenv ("VAMP_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Frameworks";
+	
+	setenv ("VAMP_PATH", path.c_str(), 1);
+
+	cstr = getenv ("ARDOUR_CONTROL_SURFACE_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Surfaces";
+	
+	setenv ("ARDOUR_CONTROL_SURFACE_PATH", path.c_str(), 1);
+
+	cstr = getenv ("LV2_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Plugins";
+	
+	setenv ("LV2_PATH", path.c_str(), 1);
+
 	path = dir_path;
 	path += "/../Frameworks/clearlooks";
 
@@ -325,13 +345,6 @@ int main (int argc, char* argv[])
 		exit (1);
 	}
 
-	if (!no_splash) {
-		ui->show_splash ();
-		if (session_name.length()) {  
-			g_timeout_add (4000, show_ui_callback, ui);
-		}
-	}
-
 	setup_keybindings (ui);
 
 	ui->run (text_receiver);
