@@ -378,12 +378,9 @@ NewSessionDialog::NewSessionDialog()
 	title += _("Session Control");
 	set_title(title.get_string());
 
-	//set_modal(false);
-	//property_window_position().set_value(Gtk::WIN_POS_NONE);
+	set_position (Gtk::WIN_POS_MOUSE);
 	set_resizable(false);
-	//property_destroy_with_parent().set_value(false);
 	set_has_separator(false);
-	// add_button(Gtk::Stock::HELP, Gtk::RESPONSE_HELP);
 	add_button(Gtk::Stock::QUIT, Gtk::RESPONSE_CANCEL);
 	add_button(Gtk::Stock::CLEAR, Gtk::RESPONSE_NONE);
 	m_okbutton = add_button(Gtk::Stock::NEW, Gtk::RESPONSE_OK);
@@ -906,7 +903,7 @@ NewSessionDialog::template_chosen ()
 void
 NewSessionDialog::recent_row_activated (const Gtk::TreePath& path, Gtk::TreeViewColumn* col)
 {
-        response (Gtk::RESPONSE_YES);
+        response (Gtk::RESPONSE_OK);
 }
 
 void
@@ -981,7 +978,7 @@ NewSessionDialog::reset_recent()
 	sessions = new std::vector<std::string*>;
 	
 	for (ARDOUR::RecentSessions::iterator i = rs.begin(); i != rs.end(); ++i) {
-	        sessions->push_back (new string ((*i).second));
+		sessions->push_back (new string ((*i).second));
 	}
 	
 	for (i = sessions->begin(); i != sessions->end(); ++i) {
@@ -993,9 +990,15 @@ NewSessionDialog::reset_recent()
 		/* remove any trailing / */
 		
 		if (fullpath[fullpath.length()-1] == '/') {
-		        fullpath = fullpath.substr (0, fullpath.length()-1);
+			fullpath = fullpath.substr (0, fullpath.length()-1);
 		}
 	    
+		/* check whether session still exists */
+		if (!Glib::file_test(fullpath, Glib::FILE_TEST_EXISTS)) {
+			/* session doesn't exist */
+			continue;
+		}		
+		
 		/* now get available states for this session */
 		  
 		if ((states = ARDOUR::Session::possible_states (fullpath)) == 0) {
