@@ -340,6 +340,7 @@ namespace ARDOUR {
 	enum PluginType {
 		AudioUnit,
 		LADSPA,
+		LV2,
 		VST
 	};
 
@@ -379,6 +380,8 @@ namespace ARDOUR {
 	    int   opts; // really RubberBandStretcher::Options
 	};
 
+	typedef std::list<nframes64_t> AnalysisFeatureList;
+
 } // namespace ARDOUR
 
 std::istream& operator>>(std::istream& o, ARDOUR::SampleFormat& sf);
