@@ -40,6 +40,7 @@
 
 #include <ardour/ardour.h>
 #include <ardour/audioengine.h>
+#include <ardour/analyser.h>
 #include <ardour/audio_diskstream.h>
 #include <ardour/utils.h>
 #include <ardour/configuration.h>
@@ -526,6 +527,11 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 
 	commit_should_unlock = false;
 
+	if (!_io->active()) {
+		_processed = true;
+		return 0;
+	}
+
 	check_record_status (transport_frame, nframes, can_record);
 
 	nominally_recording = (can_record && re);
@@ -742,6 +748,7 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 				nframes_t total = chaninfo->playback_vector.len[0] + chaninfo->playback_vector.len[1];
 				
 				if (necessary_samples > total) {
+					cerr << "underrun for " << _name << endl;
 					DiskUnderrun ();
 					goto out;
 					
@@ -825,6 +832,10 @@ AudioDiskstream::commit (nframes_t nframes)
 {
 	bool need_butler = false;
 
+	if (!_io->active()) {
+		return false;
+	}
+
 	if (_actual_speed < 0.0) {
 		playback_sample -= playback_distance;
 	} else {
@@ -1587,6 +1598,7 @@ AudioDiskstream::transport_stopped (struct tm& when, time_t twhen, bool abort_ca
 			s->update_header (capture_info.front()->start, when, twhen);
 			s->set_captured_for (_name);
 			s->mark_immutable ();
+			Analyser::queue_source_for_analysis (s, true);
 		}
 	}
 
