@@ -797,13 +797,16 @@ void
 AudioEngine::halted (void *arg)
 {
 	AudioEngine* ae = static_cast<AudioEngine *> (arg);
+	bool was_running = ae->_running;
 
 	ae->_running = false;
 	ae->_buffer_size = 0;
 	ae->_frame_rate = 0;
 	ae->_jack = 0;
 
-	ae->Halted(); /* EMIT SIGNAL */
+	if (was_running) {
+		ae->Halted(); /* EMIT SIGNAL */
+	}
 }
 
 uint32_t
@@ -1060,9 +1063,8 @@ AudioEngine::connect_to_jack (string client_name)
 	const char *server_name = NULL;
 
 	jack_client_name = client_name; /* might be reset below */
-
 	_jack = jack_client_open (jack_client_name.c_str(), options, &status, server_name);
-	
+
 	if (_jack == NULL) {
 		/* just return without an error message. something else will take care of it */
 		return -1;
