@@ -127,6 +127,9 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	_name = _current_snapshot_name = snapshot_name;
 
 	_current_frame_rate = _engine.frame_rate ();
+	_nominal_frame_rate = _current_frame_rate;
+	_base_frame_rate = _current_frame_rate;
+
 	_tempo_map = new TempoMap (_current_frame_rate);
 	_tempo_map->StateChanged.connect (mem_fun (*this, &Session::tempo_map_changed));
 
@@ -222,9 +225,6 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 		waiting_for_sync_offset = false;
 	}
 
-	_current_frame_rate = 48000;
-	_base_frame_rate = 48000;
-
 	last_smpte_when = 0;
 	_smpte_offset = 0;
 	_smpte_offset_negative = true;
@@ -469,19 +469,14 @@ Session::setup_raid_path (string path)
 }
 
 int
-Session::create (bool& new_session, const string& mix_template, nframes_t initial_length)
+Session::ensure_subdirs ()
 {
 	string dir;
 
-	if (g_mkdir_with_parents (_path.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session dir \"%1\" (%2)"), _path, strerror (errno)) << endmsg;
-		return -1;
-	}
-
 	dir = peak_dir ();
 
 	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session peakfile dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		error << string_compose(_("Session: cannot create session peakfile folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 		return -1;
 	}
 
@@ -492,7 +487,7 @@ Session::create (bool& new_session, const string& mix_template, nframes_t initia
 		dir = sound_dir ();
 		
 		if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-			error << string_compose(_("Session: cannot create session sounds dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+			error << string_compose(_("Session: cannot create session sounds folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 			return -1;
 		}
 	}
@@ -500,17 +495,39 @@ Session::create (bool& new_session, const string& mix_template, nframes_t initia
 	dir = dead_sound_dir ();
 
 	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session dead sounds dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		error << string_compose(_("Session: cannot create session dead sounds folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 		return -1;
 	}
 
 	dir = export_dir ();
 
 	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session export dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		error << string_compose(_("Session: cannot create session export folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 		return -1;
 	}
 
+	dir = analysis_dir ();
+
+	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
+		error << string_compose(_("Session: cannot create session analysis folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		return -1;
+	}
+
+	return 0;
+}
+
+int
+Session::create (bool& new_session, const string& mix_template, nframes_t initial_length)
+{
+
+	if (g_mkdir_with_parents (_path.c_str(), 0755) < 0) {
+		error << string_compose(_("Session: cannot create session folder \"%1\" (%2)"), _path, strerror (errno)) << endmsg;
+		return -1;
+	}
+
+	if (ensure_subdirs ()) {
+		return -1;
+	}
 
 	/* check new_session so we don't overwrite an existing one */
 
@@ -559,7 +576,6 @@ Session::create (bool& new_session, const string& mix_template, nframes_t initia
 
 	_state_of_the_state = Clean;
 
-
 	save_state ("");
 
 	return 0;
@@ -915,16 +931,16 @@ Session::state(bool full_state)
 
 	// store libardour version, just in case
 	char buf[16];
-	snprintf(buf, sizeof(buf)-1, "%d.%d.%d", 
-		 libardour2_major_version, libardour2_minor_version, libardour2_micro_version);
+	snprintf(buf, sizeof(buf), "%d.%d.%d", libardour2_major_version, libardour2_minor_version, libardour2_micro_version);
 	node->add_property("version", string(buf));
 		
 	/* store configuration settings */
 
 	if (full_state) {
 	
-		/* store the name */
 		node->add_property ("name", _name);
+		snprintf (buf, sizeof (buf), "%" PRId32, _nominal_frame_rate);
+		node->add_property ("sample-rate", buf);
 
 		if (session_dirs.size() > 1) {
 
@@ -1142,7 +1158,6 @@ Session::set_state (const XMLNode& node)
 	int ret = -1;
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state|CannotSave);
-
 	
 	if (node.name() != X_("Session")){
 		fatal << _("programming error: Session: incorrect XML node sent to set_state()") << endmsg;
@@ -1153,6 +1168,17 @@ Session::set_state (const XMLNode& node)
 		_name = prop->value ();
 	}
 
+	if ((prop = node.property (X_("sample-rate"))) != 0) {
+
+		_nominal_frame_rate = atoi (prop->value());
+
+		if (_nominal_frame_rate != _current_frame_rate) {
+			if (AskAboutSampleRateMismatch (_nominal_frame_rate, _current_frame_rate)) {
+				return -1;
+			}
+		}
+	}
+
 	setup_raid_path(_path);
 
 	if ((prop = node.property (X_("id-counter"))) != 0) {
@@ -2044,6 +2070,14 @@ Session::automation_dir () const
 	return res;
 }
 
+string
+Session::analysis_dir () const
+{
+	string res = _path;
+	res += "analysis/";
+	return res;
+}
+
 string
 Session::template_dir ()
 {
@@ -2105,6 +2139,10 @@ Session::template_path ()
 string
 Session::control_protocol_path ()
 {
+	char *p = getenv ("ARDOUR_CONTROL_SURFACE_PATH");
+	if (p && *p) {
+		return p;
+	}
 	return suffixed_search_path (X_("surfaces"), false);
 }
 
@@ -2808,7 +2846,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 		newpath += dead_sound_dir_name;
 
 		if (g_mkdir_with_parents (newpath.c_str(), 0755) < 0) {
-			error << string_compose(_("Session: cannot create session peakfile dir \"%1\" (%2)"), newpath, strerror (errno)) << endmsg;
+			error << string_compose(_("Session: cannot create session peakfile folder \"%1\" (%2)"), newpath, strerror (errno)) << endmsg;
 			return -1;
 		}
 
