@@ -246,29 +246,49 @@ TempoMap::~TempoMap ()
 int
 TempoMap::move_metric_section (MetricSection& section, const BBT_Time& when)
 {
-	if (when == section.start()) {
+	if (when == section.start() || !section.movable()) {
 		return -1;
 	}
 
-	if (!section.movable()) {
-		return 1;
-	}
-
 	Glib::RWLock::WriterLock  lm (lock);
 	MetricSectionSorter cmp;
-	BBT_Time corrected (when);
-	
-	if (dynamic_cast<MeterSection*>(&section) != 0) {
-		if (corrected.beats > 1) {
-			corrected.beats = 1;
-			corrected.bars++;
-		}
-	}
-	corrected.ticks = 0;
 
-	section.set_start (corrected);
-	metrics->sort (cmp);
-	timestamp_metrics ();
+	if (when.beats != 1) {
+
+		/* position by audio frame, then recompute BBT timestamps from the audio ones */
+
+		nframes_t frame = frame_time (when);
+		// cerr << "nominal frame time = " << frame << endl;
+
+		nframes_t prev_frame = round_to_type (frame, -1, Beat);
+		nframes_t next_frame = round_to_type (frame, 1, Beat);
+		
+		// cerr << "previous beat at " << prev_frame << " next at " << next_frame << endl;
+
+		/* use the closest beat */
+
+		if ((frame - prev_frame) < (next_frame - frame)) {
+			frame = prev_frame;
+		} else {
+			frame = next_frame;
+		}
+		
+		// cerr << "actual frame time = " << frame << endl;
+		section.set_frame (frame);
+		// cerr << "frame time = " << section.frame() << endl;
+		timestamp_metrics (false);
+		// cerr << "new BBT time = " << section.start() << endl;
+		metrics->sort (cmp);
+
+	} else {
+
+		/* positioned at bar start already, so just put it there */
+
+		section.set_start (when);
+		metrics->sort (cmp);
+		timestamp_metrics (true);
+	}
+
 
 	return 0;
 }
@@ -288,7 +308,6 @@ TempoMap::move_meter (MeterSection& meter, const BBT_Time& when)
 		StateChanged (Change (0));
 	}
 }
-		
 
 void
 TempoMap::remove_tempo (const TempoSection& tempo)
@@ -345,16 +364,22 @@ TempoMap::remove_meter (const MeterSection& tempo)
 }
 
 void
-TempoMap::do_insert (MetricSection* section)
+TempoMap::do_insert (MetricSection* section, bool with_bbt)
 {
 	Metrics::iterator i;
 
 	for (i = metrics->begin(); i != metrics->end(); ++i) {
 		
-		if ((*i)->start() < section->start()) {
-			continue;
+		if (with_bbt) {
+			if ((*i)->start() < section->start()) {
+				continue;
+			}
+		} else {
+			if ((*i)->frame() < section->frame()) {
+				continue;
+			}			
 		}
-		
+
 		metrics->insert (i, section);
 		break;
 	}
@@ -363,7 +388,7 @@ TempoMap::do_insert (MetricSection* section)
 		metrics->insert (metrics->end(), section);
 	}
 	
-	timestamp_metrics ();
+	timestamp_metrics (with_bbt);
 }	
 
 void
@@ -376,7 +401,18 @@ TempoMap::add_tempo (const Tempo& tempo, BBT_Time where)
 	
 		where.ticks = 0;
 		
-		do_insert (new TempoSection (where, tempo.beats_per_minute(), tempo.note_type()));
+		do_insert (new TempoSection (where, tempo.beats_per_minute(), tempo.note_type()), true);
+	}
+
+	StateChanged (Change (0));
+}
+
+void
+TempoMap::add_tempo (const Tempo& tempo, nframes_t where)
+{
+	{
+		Glib::RWLock::WriterLock lm (lock);
+		do_insert (new TempoSection (where, tempo.beats_per_minute(), tempo.note_type()), false);
 	}
 
 	StateChanged (Change (0));
@@ -395,11 +431,12 @@ TempoMap::replace_tempo (TempoSection& existing, const Tempo& replacement)
 			TempoSection *ts;
 
 			if ((ts = dynamic_cast<TempoSection*>(*i)) != 0 && ts == &existing) {
-				
-				*((Tempo *) ts) = replacement;
+
+				 *((Tempo *) ts) = replacement;
 
 				replaced = true;
-				timestamp_metrics ();
+				timestamp_metrics (true);
+
 				break;
 			}
 		}
@@ -432,7 +469,18 @@ TempoMap::add_meter (const Meter& meter, BBT_Time where)
 		
 		where.ticks = 0;
 
-		do_insert (new MeterSection (where, meter.beats_per_bar(), meter.note_divisor()));
+		do_insert (new MeterSection (where, meter.beats_per_bar(), meter.note_divisor()), true);
+	}
+
+	StateChanged (Change (0));
+}
+
+void
+TempoMap::add_meter (const Meter& meter, nframes_t where)
+{
+	{
+		Glib::RWLock::WriterLock lm (lock);
+		do_insert (new MeterSection (where, meter.beats_per_bar(), meter.note_divisor()), false);
 	}
 
 	StateChanged (Change (0));
@@ -454,7 +502,7 @@ TempoMap::replace_meter (MeterSection& existing, const Meter& replacement)
 				*((Meter*) ms) = replacement;
 
 				replaced = true;
-				timestamp_metrics ();
+				timestamp_metrics (true);
 				break;
 			}
 		}
@@ -465,6 +513,64 @@ TempoMap::replace_meter (MeterSection& existing, const Meter& replacement)
 	}
 }
 
+void
+TempoMap::change_initial_tempo (double beats_per_minute, double note_type)
+{
+	Tempo newtempo (beats_per_minute, note_type);
+	TempoSection* t;
+
+	for (Metrics::iterator i = metrics->begin(); i != metrics->end(); ++i) {
+		if ((t = dynamic_cast<TempoSection*> (*i)) != 0) {
+			*((Tempo*) t) = newtempo;
+			StateChanged (Change (0));
+			break;
+		}
+	}
+}
+
+void
+TempoMap::change_existing_tempo_at (nframes_t where, double beats_per_minute, double note_type)
+{
+	Tempo newtempo (beats_per_minute, note_type);
+
+	TempoSection* prev;
+	TempoSection* first;
+	Metrics::iterator i;
+
+	/* find the TempoSection immediately preceding "where"
+	 */
+
+	for (first = 0, i = metrics->begin(), prev = 0; i != metrics->end(); ++i) {
+
+		if ((*i)->frame() > where) {
+			break;
+		}
+
+		TempoSection* t;
+
+		if ((t = dynamic_cast<TempoSection*>(*i)) != 0) {
+			if (!first) {
+				first = t;
+			}
+			prev = t;
+		}
+	}
+
+	if (!prev) {
+		if (!first) {
+			error << string_compose (_("no tempo sections defined in tempo map - cannot change tempo @ %1"), where) << endmsg;
+			return;
+		}
+
+		prev = first;
+	}
+
+	/* reset */
+
+	*((Tempo*)prev) = newtempo;
+	StateChanged (Change (0));
+}
+
 const MeterSection&
 TempoMap::first_meter () const
 {
@@ -498,43 +604,112 @@ TempoMap::first_tempo () const
 }
 
 void
-TempoMap::timestamp_metrics ()
+TempoMap::timestamp_metrics (bool use_bbt)
 {
 	Metrics::iterator i;
 	const Meter* meter;
 	const Tempo* tempo;
 	Meter *m;
 	Tempo *t;
-	nframes_t current;
-	nframes_t section_frames;
-	BBT_Time start;
-	BBT_Time end;
 
 	meter = &first_meter ();
 	tempo = &first_tempo ();
-	current = 0;
 
-	for (i = metrics->begin(); i != metrics->end(); ++i) {
+	if (use_bbt) {
+
+		// cerr << "\n\n\n ######################\nTIMESTAMP via BBT ##############\n" << endl;
+
+		nframes_t current = 0;
+		nframes_t section_frames;
+		BBT_Time start;
+		BBT_Time end;
+
+		for (i = metrics->begin(); i != metrics->end(); ++i) {
+			
+			end = (*i)->start();
+			
+			section_frames = count_frames_between_metrics (*meter, *tempo, start, end);
+			
+			current += section_frames;
+			
+			start = end;
+			
+			(*i)->set_frame (current);
+			
+			if ((t = dynamic_cast<TempoSection*>(*i)) != 0) {
+				tempo = t;
+			} else if ((m = dynamic_cast<MeterSection*>(*i)) != 0) {
+				meter = m;
+			} else {
+				fatal << _("programming error: unhandled MetricSection type") << endmsg;
+				/*NOTREACHED*/
+			}
+		}
+
+	} else {
+
+		// cerr << "\n\n\n ######################\nTIMESTAMP via AUDIO ##############\n" << endl;
+
+		bool first = true;
+		MetricSection* prev = 0;
+
+		for (i = metrics->begin(); i != metrics->end(); ++i) {
+
+			BBT_Time bbt;
+			Metric metric (*meter, *tempo);
+			
+			if (prev) {
+				metric.set_start (prev->start());
+			} else {
+				// metric will be at frames=0 bbt=1|1|0 by default
+				// which is correct for our purpose
+			}
 		
-		end = (*i)->start();
+			bbt_time_with_metric ((*i)->frame(), bbt, metric);
 
-		section_frames = count_frames_between_metrics (*meter, *tempo, start, end);
+			// cerr << "timestamp @ " << (*i)->frame() << " with " << bbt.bars << "|" << bbt.beats << "|" << bbt.ticks << " => ";
+			
 
-		current += section_frames;
+			if (first) {
+				first = false;
+			} else {
+				
+				if (bbt.ticks > Meter::ticks_per_beat/2) {
+					/* round up to next beat */
+					bbt.beats += 1;
+				} 
 
-		start = end;
+				bbt.ticks = 0;
 
-		(*i)->set_frame (current);
+				if (bbt.beats != 1) {
+					/* round up to next bar */
+					bbt.bars += 1;
+					bbt.beats = 1;
+				}
+			}
+			
+			//s cerr << bbt.bars << "|" << bbt.beats << "|" << bbt.ticks << endl;
+			
+			(*i)->set_start (bbt);
 
-		if ((t = dynamic_cast<TempoSection*>(*i)) != 0) {
-			tempo = t;
-		} else if ((m = dynamic_cast<MeterSection*>(*i)) != 0) {
-			meter = m;
-		} else {
-			fatal << _("programming error: unhandled MetricSection type") << endmsg;
-			/*NOTREACHED*/
+			if ((t = dynamic_cast<TempoSection*>(*i)) != 0) {
+				tempo = t;
+				// cerr << "NEW TEMPO, frame = " << (*i)->frame() << " start = " << (*i)->start() <<endl;
+			} else if ((m = dynamic_cast<MeterSection*>(*i)) != 0) {
+				meter = m;
+				// cerr << "NEW METER, frame = " << (*i)->frame() << " start = " << (*i)->start() <<endl;
+			} else {
+				fatal << _("programming error: unhandled MetricSection type") << endmsg;
+				/*NOTREACHED*/
+			}
+
+			prev = (*i);
 		}
 	}
+
+	// dump (cerr);
+	// cerr << "###############################################\n\n\n" << endl;
+
 }
 
 TempoMap::Metric
@@ -629,6 +804,8 @@ TempoMap::bbt_time_with_metric (nframes_t frame, BBT_Time& bbt, const Metric& me
 	double xtra_beats = 0;
 	double beats = 0;
 
+	// cerr << "---- BBT time for " << frame << " using metric @ " << metric.frame() << " BBT " << metric.start() << endl;
+
 	const double beats_per_bar = metric.meter().beats_per_bar();
 	const double frames_per_bar = metric.meter().frames_per_bar (metric.tempo(), _frame_rate);
 	const double beat_frames = metric.tempo().frames_per_beat (_frame_rate, metric.meter());
@@ -636,11 +813,14 @@ TempoMap::bbt_time_with_metric (nframes_t frame, BBT_Time& bbt, const Metric& me
 	/* now compute how far beyond that point we actually are. */
 
 	frame_diff = frame - metric.frame();
+	
+	// cerr << "----\tdelta = " << frame_diff << endl;
 
 	xtra_bars = (uint32_t) floor (frame_diff / frames_per_bar);
 	frame_diff -= (uint32_t) floor (xtra_bars * frames_per_bar);
 	xtra_beats = (double) frame_diff / beat_frames;
 
+	// cerr << "---\tmeaning " << xtra_bars << " xtra bars and " << xtra_beats << " xtra beats\n";
 
 	/* and set the returned value */
 
@@ -653,30 +833,28 @@ TempoMap::bbt_time_with_metric (nframes_t frame, BBT_Time& bbt, const Metric& me
 	bbt.bars = metric.start().bars + xtra_bars; 
 
 	beats = (double) metric.start().beats + xtra_beats;
-
+	
 	bbt.bars += (uint32_t) floor(beats/ (beats_per_bar+1) );
 
 	beats = fmod(beats - 1, beats_per_bar )+ 1.0;
 	bbt.ticks = (uint32_t)( round((beats - floor(beats)) *(double) Meter::ticks_per_beat));
 	bbt.beats = (uint32_t) floor(beats);
-
+	
+	// cerr << "-----\t RETURN " << bbt << endl;
 }
 
-
 nframes_t 
 TempoMap::count_frames_between ( const BBT_Time& start, const BBT_Time& end) const
 {
-
-        /* for this to work with fractional measure types, start and end have to "legal" BBT types, 
-        that means that  the  beats and ticks should be  inside a bar
+        /* for this to work with fractional measure types, start and end have to be "legal" BBT types, 
+	   that means that the beats and ticks should be inside a bar
 	*/
 
-
 	nframes_t frames = 0;
 	nframes_t start_frame = 0;
 	nframes_t end_frame = 0;
 
-	Metric m = metric_at(start);
+	Metric m = metric_at (start);
 
 	uint32_t bar_offset = start.bars - m.start().bars;
 
@@ -704,7 +882,7 @@ TempoMap::count_frames_between ( const BBT_Time& start, const BBT_Time& end) con
 nframes_t 
 TempoMap::count_frames_between_metrics (const Meter& meter, const Tempo& tempo, const BBT_Time& start, const BBT_Time& end) const
 {
-        /*this is used in timestamping the metrics by actually counting the beats */ 
+        /* this is used in timestamping the metrics by actually counting the beats */ 
 
 	nframes_t frames = 0;
 	uint32_t bar = start.bars;
@@ -724,16 +902,26 @@ TempoMap::count_frames_between_metrics (const Meter& meter, const Tempo& tempo,
 			beat = 1;
 			++bar;
 			++beats_counted;
+
+			if (beat > beats_per_bar) {
+
+				/* this is a fractional beat at the end of a fractional bar
+				   so it should only count for the fraction 
+				*/
+
+				beats_counted -= (ceil(beats_per_bar) - beats_per_bar);
+			}
+
 		} else {
 			++beat;
 			++beats_counted;
-			if (beat > beats_per_bar) {
-				/* this is a fractional beat at the end of a fractional bar
-				   so it should only count for the fraction */
-				beats_counted -= (ceil(beats_per_bar) - beats_per_bar);
-			}
 		}
 	}
+
+	// cerr << "Counted " << beats_counted << " from " << start << " to " << end 
+	// << " bpb were " << beats_per_bar 
+	// << " fpb was " << beat_frames
+	// << endl;
 	
 	frames = (nframes_t) floor (beats_counted * beat_frames);
 
@@ -939,71 +1127,9 @@ TempoMap::round_to_beat_subdivision (nframes_t fr, int sub_num)
 	}
 
 	return frame_time (the_beat);
-
-
-
-	/*****************************  
-	XXX just keeping this for reference
-
-        TempoMap::BBTPointList::iterator i;
-        TempoMap::BBTPointList *more_zoomed_bbt_points;
-        nframes_t frame_one_beats_worth;
-        nframes_t pos = 0;
-	nframes_t next_pos = 0 ;
-        double tempo = 1;
-        double frames_one_subdivisions_worth;
-        bool fr_has_changed = false;
-
-        int n;
-
-	frame_one_beats_worth = (nframes_t) ::floor ((double)  _frame_rate *  60 / 20 ); //one beat @ 20 bpm
-        {
-	  Glib::RWLock::ReaderLock lm (lock);
-	  more_zoomed_bbt_points = get_points((fr >= frame_one_beats_worth) ? 
-					    fr - frame_one_beats_worth : 0, fr+frame_one_beats_worth );
-	}
-	if (more_zoomed_bbt_points == 0 || more_zoomed_bbt_points->empty()) {
-		return fr;
-	}
-
-	for (i = more_zoomed_bbt_points->begin(); i != more_zoomed_bbt_points->end(); i++) {
-		if  ((*i).frame <= fr) {
-			pos = (*i).frame;
-			tempo = (*i).tempo->beats_per_minute();
-			
-		} else {
-			i++;
-			next_pos = (*i).frame;
-			break;
-		}
-	}
-	frames_one_subdivisions_worth = ((double) _frame_rate *  60 / (sub_num * tempo));
-
-	for (n = sub_num; n > 0; n--) {
-		if (fr >= (pos + ((n - 0.5) * frames_one_subdivisions_worth))) {
-			fr = (nframes_t) round(pos + (n  * frames_one_subdivisions_worth));
-		 	if (fr > next_pos) {
- 				fr = next_pos;  //take care of fractional beats that don't match the subdivision asked
- 			}
-			fr_has_changed = true;
-			break;
-		}
-	}
-
-	if (!fr_has_changed) {
-		fr = pos;
-	}
-
-        delete more_zoomed_bbt_points;
-        return fr ;
-
-	******************************/
-
-
 }
 
 nframes_t
-
 TempoMap::round_to_type (nframes_t frame, int dir, BBTPointType type)
 {
 	Metric metric = metric_at (frame);
@@ -1051,6 +1177,12 @@ TempoMap::round_to_type (nframes_t frame, int dir, BBTPointType type)
 	
 	}
 
+	/* 
+	   cerr << "for " << frame << " round to " << bbt << " using "
+	   << metric.start()
+	   << endl;
+	*/
+
 	return metric.frame() + count_frames_between (metric.start(), bbt);
 }
 
@@ -1136,7 +1268,9 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 
 		if (i == metrics->end()) {
 			limit = upper;
+			// cerr << "== limit set to end of request @ " << limit << endl;
 		} else {
+ 			// cerr << "== limit set to next metric @ " << (*i)->frame() << endl;
 			limit = (*i)->frame();
 		}
 
@@ -1148,6 +1282,7 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 
 			if (beat == 1) {
 				if (current >= lower) {
+					// cerr << "Add Bar at " << bar << "|1" << " @ " << current << endl;
 					points->push_back (BBTPoint (*meter, *tempo,(nframes_t)rint(current), Bar, bar, 1));
 
 				}
@@ -1159,6 +1294,7 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 
 			while (beat <= ceil( beats_per_bar) && beat_frame < limit) {
 				if (beat_frame >= lower) {
+					// cerr << "Add Beat at " << bar << '|' << beat << " @ " << beat_frame << endl;
 					points->push_back (BBTPoint (*meter, *tempo, (nframes_t) rint(beat_frame), Beat, bar, beat));
 				}
 				beat_frame += beat_frames;
@@ -1167,7 +1303,11 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 				beat++;
 			}
 
-			if (beat > ceil(beats_per_bar) ) {
+			//  cerr << "out of beats, @ end ? " << (i == metrics->end()) << " out of bpb ? "
+			// << (beat > ceil(beats_per_bar))
+			// << endl;
+
+			if (beat > ceil(beats_per_bar) || i != metrics->end()) {
 
 				/* we walked an entire bar. its
 				   important to move `current' forward
@@ -1185,10 +1325,17 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 				   so we subtract the possible extra fraction from the current
 				*/
 
-				current -=  beat_frames * (ceil(beats_per_bar)-beats_per_bar);
+				if (beat > ceil (beats_per_bar)) {
+					/* next bar goes where the numbers suggest */
+					current -=  beat_frames * (ceil(beats_per_bar)-beats_per_bar);
+					// cerr << "++ next bar from numbers\n";
+				} else {
+					/* next bar goes where the next metric is */
+					current = limit;
+					// cerr << "++ next bar at next metric\n";
+				}
 				bar++;
 				beat = 1;
-
 			} 
 		
 		}
@@ -1213,6 +1360,9 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 				beat = 1;
 			}
 
+			current = (*i)->frame ();
+			// cerr << "loop around with current @ " << current << endl;
+
 			beats_per_bar = meter->beats_per_bar ();
 			frames_per_bar = meter->frames_per_bar (*tempo, _frame_rate);
 			beat_frames = tempo->frames_per_beat (_frame_rate, *meter);
@@ -1225,6 +1375,33 @@ TempoMap::get_points (nframes_t lower, nframes_t upper) const
 	return points;
 }	
 
+const TempoSection&
+TempoMap::tempo_section_at (nframes_t frame)
+{
+	Glib::RWLock::ReaderLock lm (lock);
+	Metrics::iterator i;
+	TempoSection* prev = 0;
+	
+	for (i = metrics->begin(); i != metrics->end(); ++i) {
+		TempoSection* t;
+
+		if ((t = dynamic_cast<TempoSection*> (*i)) != 0) {
+
+			if ((*i)->frame() > frame) {
+				break;
+			}
+
+			prev = t;
+		}
+	}
+
+	if (prev == 0) {
+		fatal << endmsg;
+	}
+
+	return *prev;
+}
+
 const Tempo&
 TempoMap::tempo_at (nframes_t frame)
 {
@@ -1303,7 +1480,7 @@ TempoMap::set_state (const XMLNode& node)
 			
 			MetricSectionSorter cmp;
 			metrics->sort (cmp);
-			timestamp_metrics ();
+			timestamp_metrics (true);
 		}
 	}
 	
@@ -1330,3 +1507,32 @@ TempoMap::dump (std::ostream& o) const
 	}
 }
 
+int
+TempoMap::n_tempos() const
+{
+	Glib::RWLock::ReaderLock lm (lock);
+	int cnt = 0;
+
+	for (Metrics::const_iterator i = metrics->begin(); i != metrics->end(); ++i) {
+		if (dynamic_cast<const TempoSection*>(*i) != 0) {
+			cnt++;
+		}
+	}
+
+	return cnt;
+}
+
+int
+TempoMap::n_meters() const
+{
+	Glib::RWLock::ReaderLock lm (lock);
+	int cnt = 0;
+
+	for (Metrics::const_iterator i = metrics->begin(); i != metrics->end(); ++i) {
+		if (dynamic_cast<const MeterSection*>(*i) != 0) {
+			cnt++;
+		}
+	}
+
+	return cnt;
+}
