@@ -27,7 +27,6 @@
 
 #include <alsa/asoundlib.h>
 #include <midi++/port.h>
-#include <midi++/port_request.h>
 
 namespace MIDI {
 
@@ -35,7 +34,7 @@ class ALSA_SequencerMidiPort : public Port
 
 {
   public:
-	ALSA_SequencerMidiPort (PortRequest &req);
+	ALSA_SequencerMidiPort (const XMLNode&);
 	virtual ~ALSA_SequencerMidiPort ();
 
 	/* select(2)/poll(2)-based I/O */
@@ -45,6 +44,9 @@ class ALSA_SequencerMidiPort : public Port
 	static int discover (std::vector<PortSet>&);
 	static std::string typestring;
 
+	XMLNode& get_state() const;
+	void set_state (const XMLNode&);
+
   protected:
 	/* Direct I/O */
 	
@@ -60,10 +62,13 @@ class ALSA_SequencerMidiPort : public Port
 	int port_id;
 	snd_seq_event_t SEv;
 
-	int CreatePorts(PortRequest &req);
+	int create_ports (const Port::Descriptor&);
 
 	static int init_client (std::string name);
 	static snd_seq_t* seq;
+
+	typedef std::pair<int,int> SequencerPortAddress;
+	void get_connections (std::vector<SequencerPortAddress>&, int dir) const;
 };
 
 }; /* namespace MIDI */
